<?php declare(strict_types = 0);
/*
** Zabbix
** Copyright (C) 2001-2023 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
**/


namespace Modules\HostGroupAutoPerm\Actions;

use API,
	APP,
	CControllerResponseData,
	RuntimeException;

class CControllerHostGroupCreate extends \CControllerHostGroupCreate {

	protected function doAction(): void {
		DBstart();

		$result = API::HostGroup()->create(['name' => $this->getInput('name')]);

		$output = [];

		if ($result) {
			$output['success']['title'] = _('Host group added');

			if ($messages = get_and_clear_messages()) {
				$output['success']['messages'] = array_column($messages, 'message');
			}

			try {
				$user_groups_names = $this->updateHostGroupPermissions($result['groupids'][0]);

				if ($user_groups_names) {
					$output['success']['messages'][] = 'HostGroupAutoPerm: Read permission added to user groups: "'.
						implode('", "', $user_groups_names).'".';
				}

				DBend();
			}
			catch (\RuntimeException $e) {
				$output = [
					'error' => [
						'title' => _('Cannot add host group'),
						'messages' => [
							'HostGroupAutoPerm error: '.$e->getMessage()
						]
					]
				];

				DBend(false);
			}
		}
		else {
			$output['error'] = [
				'title' => _('Cannot add host group'),
				'messages' => array_column(get_and_clear_messages(), 'message')
			];

			DBend(false);
		}

		$this->setResponse((new CControllerResponseData(['main_block' => json_encode($output)]))->disableView());
	}

	private function updateHostGroupPermissions($groupid): array {
		$module = APP::ModuleManager()->getActionModule();
		$user_groups_names = $module->getOption('user_groups_names');

		if (!is_array($user_groups_names)) {
			return [];
		}

		$db_user_groups = API::UserGroup()->get([
			'output' => ['usrgrpid'],
			'selectHostGroupRights' => ['id', 'permission'],
			'search' => $user_groups_names ? ['name' => $user_groups_names] : null
		]);

		if ($user_groups_names && count($db_user_groups) != count($user_groups_names)) {
			throw new RuntimeException('Cannot read user groups: "'.implode('", "', $user_groups_names).'".');
		}

		foreach ($db_user_groups as &$user_group) {
			$user_group['hostgroup_rights'][] = [
				'id' => $groupid,
				'permission' => PERM_READ
			];
		}
		unset($user_group);

		$result = API::UserGroup()->update($db_user_groups);

		if (!$result) {
			throw new RuntimeException('Cannot update user groups: "'.implode('", "', $user_groups_names).'".');
		}

		return $user_groups_names;
	}
}
